@@ -9,7 +9,6 @@ import logging
 import os
 import re
 
-import salt.auth.ldap
 import salt.cache
 import salt.payload
 import salt.roster
@@ -109,11 +108,11 @@ def get_minion_data(minion, opts):
         cache = salt.cache.factory(opts)
         if minion is None:
             for id_ in cache.list("minions"):
-                data = cache.fetch("minions/{}".format(id_), "data")
+                data = cache.fetch(f"minions/{id_}", "data")
                 if data is None:
                     continue
         else:
-            data = cache.fetch("minions/{}".format(minion), "data")
+            data = cache.fetch(f"minions/{minion}", "data")
         if data is not None:
             grains = data.get("grains", None)
             pillar = data.get("pillar", None)
@@ -216,6 +215,10 @@ class CkMinions:
             self.acc = "minions"
         else:
             self.acc = "accepted"
+        if self.opts.get("cluster_id", None) is not None:
+            self.pki_dir = self.opts.get("cluster_pki_dir", "")
+        else:
+            self.pki_dir = self.opts.get("pki_dir", "")
 
     def _check_nodegroup_minions(self, expr, greedy):  # pylint: disable=unused-argument
         """
@@ -257,11 +260,11 @@ class CkMinions:
 
     def _pki_minions(self):
         """
-        Retreive complete minion list from PKI dir.
+        Retrieve complete minion list from PKI dir.
         Respects cache if configured
         """
         minions = []
-        pki_cache_fn = os.path.join(self.opts["pki_dir"], self.acc, ".key_cache")
+        pki_cache_fn = os.path.join(self.pki_dir, self.acc, ".key_cache")
         try:
             os.makedirs(os.path.dirname(pki_cache_fn))
         except OSError:
@@ -273,11 +276,9 @@ class CkMinions:
                     return salt.payload.load(fn_)
             else:
                 for fn_ in salt.utils.data.sorted_ignorecase(
-                    os.listdir(os.path.join(self.opts["pki_dir"], self.acc))
+                    os.listdir(os.path.join(self.pki_dir, self.acc))
                 ):
-                    if not fn_.startswith(".") and os.path.isfile(
-                        os.path.join(self.opts["pki_dir"], self.acc, fn_)
-                    ):
+                    if not fn_.startswith("."):
                         minions.append(fn_)
             return minions
         except OSError as exc:
@@ -303,11 +304,9 @@ class CkMinions:
         if greedy:
             minions = []
             for fn_ in salt.utils.data.sorted_ignorecase(
-                os.listdir(os.path.join(self.opts["pki_dir"], self.acc))
+                os.listdir(os.path.join(self.pki_dir, self.acc))
             ):
-                if not fn_.startswith(".") and os.path.isfile(
-                    os.path.join(self.opts["pki_dir"], self.acc, fn_)
-                ):
+                if not fn_.startswith("."):
                     minions.append(fn_)
         elif cache_enabled:
             minions = list_cached_minions()
@@ -325,7 +324,7 @@ class CkMinions:
             for id_ in cminions:
                 if greedy and id_ not in minions:
                     continue
-                mdata = self.cache.fetch("minions/{}".format(id_), "data")
+                mdata = self.cache.fetch(f"minions/{id_}", "data")
                 if mdata is None:
                     if not greedy:
                         minions.remove(id_)
@@ -410,11 +409,11 @@ class CkMinions:
                 except Exception:  # pylint: disable=broad-except
                     log.error("Invalid IP/CIDR target: %s", tgt)
                     return {"minions": [], "missing": []}
-            proto = "ipv{}".format(tgt.version)
+            proto = f"ipv{tgt.version}"
 
             minions = set(minions)
             for id_ in cminions:
-                mdata = self.cache.fetch("minions/{}".format(id_), "data")
+                mdata = self.cache.fetch(f"minions/{id_}", "data")
                 if mdata is None:
                     if not greedy:
                         minions.remove(id_)
@@ -451,11 +450,9 @@ class CkMinions:
             if greedy:
                 mlist = []
                 for fn_ in salt.utils.data.sorted_ignorecase(
-                    os.listdir(os.path.join(self.opts["pki_dir"], self.acc))
+                    os.listdir(os.path.join(self.pki_dir, self.acc))
                 ):
-                    if not fn_.startswith(".") and os.path.isfile(
-                        os.path.join(self.opts["pki_dir"], self.acc, fn_)
-                    ):
+                    if not fn_.startswith("."):
                         mlist.append(fn_)
                 return {"minions": mlist, "missing": []}
             elif cache_enabled:
@@ -651,7 +648,7 @@ class CkMinions:
                 search = subset
             for id_ in search:
                 try:
-                    mdata = self.cache.fetch("minions/{}".format(id_), "data")
+                    mdata = self.cache.fetch(f"minions/{id_}", "data")
                 except SaltCacheError:
                     # If a SaltCacheError is explicitly raised during the fetch operation,
                     # permission was denied to open the cached data.p file. Continue on as
@@ -683,11 +680,9 @@ class CkMinions:
         """
         mlist = []
         for fn_ in salt.utils.data.sorted_ignorecase(
-            os.listdir(os.path.join(self.opts["pki_dir"], self.acc))
+            os.listdir(os.path.join(self.pki_dir, self.acc))
         ):
-            if not fn_.startswith(".") and os.path.isfile(
-                os.path.join(self.opts["pki_dir"], self.acc, fn_)
-            ):
+            if not fn_.startswith("."):
                 mlist.append(fn_)
         return {"minions": mlist, "missing": []}
 
@@ -704,7 +699,7 @@ class CkMinions:
         try:
             if expr is None:
                 expr = ""
-            check_func = getattr(self, "_check_{}_minions".format(tgt_type), None)
+            check_func = getattr(self, f"_check_{tgt_type}_minions", None)
             if tgt_type in (
                 "grain",
                 "grain_pcre",
@@ -748,16 +743,15 @@ class CkMinions:
         """
 
         v_minions = set(self.check_minions(valid, "compound").get("minions", []))
-        if not v_minions:
-            # There are no valid minions, so it doesn't matter what we are
-            # targeting - this is a fail.
-            return False
         if minions is None:
             _res = self.check_minions(expr, tgt_type)
             minions = set(_res["minions"])
         else:
             minions = set(minions)
-        return minions.issubset(v_minions)
+        d_bool = not bool(minions.difference(v_minions))
+        if len(v_minions) == len(minions) and d_bool:
+            return True
+        return d_bool
 
     def match_check(self, regex, fun):
         """
@@ -1051,11 +1045,7 @@ class CkMinions:
         for ind in auth_list:
             if isinstance(ind, str):
                 if ind[0] == "@":
-                    if (
-                        ind[1:] == mod_name
-                        or ind[1:] == form
-                        or ind == "@{}s".format(form)
-                    ):
+                    if ind[1:] == mod_name or ind[1:] == form or ind == f"@{form}s":
                         return True
             elif isinstance(ind, dict):
                 if len(ind) != 1:
@@ -1067,7 +1057,7 @@ class CkMinions:
                             ind[valid], fun_name, args.get("arg"), args.get("kwarg")
                         ):
                             return True
-                    if valid[1:] == form or valid == "@{}s".format(form):
+                    if valid[1:] == form or valid == f"@{form}s":
                         if self.__fun_check(
                             ind[valid], fun, args.get("arg"), args.get("kwarg")
                         ):
